from dataclasses import dataclass, field
import osmnx as ox
import pandas as pd

@dataclass
class OSM_geometry:
    """
        Point of interest in lat, long format that retrieves Open 
        Street Maps's features

        Args:
            lat: center point latitude.
            lon: center point longitude.
            radio: distance in meters to cover from the center point.
    """
    center_point: int
    tags: dict
    radio: int = field(default=1000)

    def get_features_from_point(self):
        features = ox.features.features_from_point(
            self.center_point,
            dist=self.radio,
            tags=self.tags
        )
        return features

    def get_graph_from_point(self):
        graph = ox.graph.graph_from_point(
            self.center_point,
            dist=self.radio
        )
        return graph

    def get_basic_stats(self):
        return pd.DataFrame(
            ox.stats.basic_stats(self.graph)
            .drop(
                columns=[
                    "streets_per_node_proportions",
                    "streets_per_node_counts"
                    ]
            )  # we dont wanna use this columns at the moment
            .drop_duplicates()
            # delete repeated rows due to the columns droped above
            )

    def __post_init__(self):
        self.graph = self.get_graph_from_point()
        self.features = self.get_features_from_point()
        self.statistics = self.get_basic_stats()

    def create_tag_value_counts(self):
        dfs = []
        for tag in self.tags.keys():
            def rename_columns(df):
                df.columns = [f"{tag}_{c}" for c in df.columns]
                return df
            dfs.append(
                self.features
                [tag]
                .value_counts()
                .rename("center_point_data")
                .to_frame()
                .T
                .pipe(rename_columns)
            )
        return pd.concat(dfs, axis=1)



import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
import wget
import os

# TODO:
# 1: CHECK IF 18S OR 19S SHOULD APPLY


def get_census_dataset() -> gpd.GeoDataFrame:
    os.makedirs("data", exist_ok=True)
    file = wget.download("https://github.com/martin-conur/geo_utils/blob/main/geo_utils/data/info_manzanas_censo2017.feather?raw=True", out="data/info_manzanas_censo2017.feather")
    return gpd.read_feather(file)


def get_ismt_dataset() -> gpd.GeoDataFrame:
    os.makedirs("data", exist_ok=True)
    file = wget.download("https://github.com/martin-conur/geo_utils/blob/main/geo_utils/data/ISMT_2022_actualizado_simplificado.feather?raw=True", out="data/ISMT_2022_actualizado_simplificado.feather")
    return gpd.read_feather(file)


def get_coverture_percentage(df, other):
    return (
        df
        .assign(
            coverture=df.area / other.area
        )
    )


def get_census_data_from_point(lat: float,
                               lon: float,
                               census_df: gpd.GeoDataFrame,
                               radio: int = 1000) -> pd.DataFrame:
    # defining the geoseries point from 'lat' and 'lon'
    point = (
        gpd.GeoSeries(Point(lon, lat), crs=4326)
        .to_crs(9155)  # TODO 1
        .buffer(1000)
        .to_crs(4326)
    )

    # Creating the filtered dataframe
    def weighted_sum_censo(df):
        coverture = df.pop("coverture")
        return df.mul(coverture, axis=0).sum()

    censo_intersection = census_df.intersects(point.geometry.iloc[0])
    censo_columns = ["TOTAL_VIVI", "NHOMBRES",	"NMUJERES", "coverture"]
    inter_censo = (
        census_df
        .assign(
            geometry=census_df.intersection(point.iloc[0])
        )
        .pipe(get_coverture_percentage, census_df)
        .loc[censo_intersection]
        [censo_columns]
        .pipe(weighted_sum_censo)
        .to_frame()
        .T
    )
    return inter_censo


def get_ismt_data_from_point(lat: float,
                             lon: float,
                             ismt_df: gpd.GeoDataFrame,
                             radio: int = 1000) -> pd.DataFrame:

    # defining the geoseries point from 'lat' and 'lon'
    point = (
        gpd.GeoSeries(Point(lon, lat), crs=4326)
        .to_crs(9155)  # TODO 1
        .buffer(1000)
        .to_crs(4326)
    )
    # Creating the filtered dataframe
    ismt_intersection = ismt_df.intersects(point.geometry.iloc[0])
    columns = [
        'Alto',
        'Medio',
        'Bajo',
        'AB',
        'C1',
        'C2',
        'C3',
        'D',
        'E',
        'ind_hac',
        'alleg',
        'escolar',
        'ind_mat',
        'coverture'
        ]

    inter = (
        ismt_df
        .assign(
            geometry=ismt_df.intersection(point.geometry.iloc[0])
        )
        .pipe(get_coverture_percentage, ismt_df)
        .loc[ismt_intersection]
        [columns]
    )
    coverture = inter.pop("coverture")

    weighted_sum = inter.mul(coverture, axis=0).sum()
    inter_mean = (
        weighted_sum
        .div(coverture.sum())
        [
            ["escolar", "ind_hac", "ind_mat"]
        ]
        .to_frame()
        .T
    )

    inter_sum = (
        weighted_sum
        [
            [
                'Alto',
                'Medio',
                'Bajo',
                'AB',
                'C1',
                'C2',
                'C3',
                'D',
                'E',
                'alleg'
            ]
        ]
        .to_frame()
        .T
    )
    return pd.concat([inter_mean, inter_sum], axis=1)



from setuptools import setup

setup(
    name='mcu-geo-utils',
    version='0.1',
    author='Martin Conur',
    author_email='martincontrerasur@gmail.com',
    packages=['mcu_geo_utils'],
    scripts=[],
   # url='http://pypi.python.org/pypi/apiweather/',
    license='MIT',
    description='variaty of geo-related tools',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "pandas >= 1.5.1",
        "geopandas == 0.12.1",
        "shapely == 2.0.1",
        "osmnx == 1.5.1"
    ],
)